#!/usr/bin/perl

use strict;
use warnings;
use 5.0008;

use File::Basename;

# Generate a VMDK which points at a flat image

sub get_length {
	my $filename = shift;
	my @stats = stat($filename);
	return $stats[7];
}

sub gen_content_id {
	# 32 hex digits total
	# 24 hex digits, then should end in fffffffe
	
	my $str = join('', map { ("a".."f", 0..9)[rand 16] } 1..24);
	$str .= "fffffffe";

	return $str;
}

sub get_uuid_part {
	return join(' ', map { ('a'..'f', 0..9)[rand 16] . ('a'..'f', 0..9)[rand 16] } 1..8);
}

sub gen_uuid {
	# format is "xx xx xx xx xx xx xx xx-xx xx xx xx xx xx xx xx"
	# where xx is a single byte (hex formatted)
	return get_uuid_part() . '-' . get_uuid_part();
}

sub calc_chs {
	my $length = shift;

	my ($heads, $sectors, $cylinders);

	if ($length < 1073741824) {
		# <1GB
		$heads = 64;
		$sectors = 32;
		$cylinders = int($length / 1048576);

	} elsif ($length < 2147483648) {
		# >1GB, <2GB
		$heads = 128;
		$sectors = 32;
		$cylinders = int($length / 2097152);
	} else {
		# >2GB
		$heads = 255;
		$sectors = 63;
		$cylinders = int($length / 8225280);
	}

	return ($cylinders, $heads, $sectors);
}

sub gen_vmdk {
	my $in_filename = shift;
	my $dirname = shift;

	my $length_in_bytes = get_length("$dirname/$in_filename");
	my $len = int($length_in_bytes / 512);
	my ($cylinders, $heads, $sectors) = calc_chs($length_in_bytes);

	my $longContentID = gen_content_id();
	my $uuid = gen_uuid();

	my $adapter_type = 'lsilogic'; # TODO support IDE and SCSI

	my $outfile;

	open OUTFILE, ">$dirname/$in_filename.vmdk" or die $!;
	print OUTFILE <<END;
# Disk Descriptor File
# generated by genvmdk

version=1
CID=fffffffe
parentCID=ffffffff
createType="vmfs"

# Extents
RW $len VMFS "$in_filename"

# DDB
ddb.virtualHWVersion = "7"
ddb.longContentID = "$longContentID"
ddb.uuid = "$uuid"
ddb.adapterType = "$adapter_type"
ddb.thinProvisioned = "0"
ddb.geometry.sectors = "$sectors"
ddb.geometry.heads = "$heads"
ddb.geometry.cylinders = "$cylinders"
END

	# Will only work on ESX/ESXi 4.x+ since virtualHWVersion is 7.
	
	close OUTFILE;
}

sub print_usage {
	print STDERR "genvmdk <filename>";
}

sub main {
	my $argc = scalar(@ARGV);

	if ($argc < 1) {
		print_usage();
		exit(1);
	}

	umask 0022;

	my $in_filename = shift @ARGV;
	my $dirname = dirname($in_filename);
	$in_filename = basename($in_filename);

	gen_vmdk($in_filename, $dirname);
}

main();
